/*
	RackAFX(TM)
	Applications Programming Interface
	Derived Class Object Definition
	Copyright(c) Tritone Systems Inc. 2006-2012

	Your plug-in must implement the constructor,
	destructor and virtual Plug-In API Functions below.
*/

#pragma once

// base class
#include "plugin.h"
#include "synthfunctions.h"

class CPhaseDistortionSynthesis : public CPlugIn
{
public:
	// RackAFX Plug-In API Member Methods:
	// The followung 5 methods must be impelemented for a meaningful Plug-In
	//
	// 1. One Time Initialization
	CPhaseDistortionSynthesis();

	// 2. One Time Destruction
	virtual ~CPhaseDistortionSynthesis(void);

	// 3. The Prepare For Play Function is called just before audio streams
	virtual bool __stdcall prepareForPlay();

	// 4. processAudioFrame() processes an audio input to create an audio output
	virtual bool __stdcall processAudioFrame(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels);

	// 5. userInterfaceChange() occurs when the user moves a control.
	virtual bool __stdcall userInterfaceChange(int nControlIndex);


	// OPTIONAL ADVANCED METHODS ------------------------------------------------------------------------------------------------
	// These are more advanced; see the website for more details
	//
	// 6. initialize() is called once just after creation; if you need to use Plug-In -> Host methods
	//				   such as sendUpdateGUI(), you must do them here and NOT in the constructor
	virtual bool __stdcall initialize();

	// 7. joystickControlChange() occurs when the user moves a control.
	virtual bool __stdcall joystickControlChange(float fControlA, float fControlB, float fControlC, float fControlD, float fACMix, float fBDMix);

	// 8. process buffers instead of Frames:
	// NOTE: set m_bWantBuffers = true to use this function
	virtual bool __stdcall processRackAFXAudioBuffer(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels, UINT uBufferSize);

	// 9. rocess buffers instead of Frames:
	// NOTE: set m_bWantVSTBuffers = true to use this function
	virtual bool __stdcall processVSTAudioBuffer(float** inBuffer, float** outBuffer, UINT uNumChannels, int inFramesToProcess);

	// 10. MIDI Note On Event
	virtual bool __stdcall midiNoteOn(UINT uChannel, UINT uMIDINote, UINT uVelocity);

	// 11. MIDI Note Off Event
	virtual bool __stdcall midiNoteOff(UINT uChannel, UINT uMIDINote, UINT uVelocity, bool bAllNotesOff);


	// 12. MIDI Modulation Wheel uModValue = 0 -> 127
	virtual bool __stdcall midiModWheel(UINT uChannel, UINT uModValue);

	// 13. MIDI Pitch Bend
	//					nActualPitchBendValue = -8192 -> 8191, 0 is center, corresponding to the 14-bit MIDI value
	//					fNormalizedPitchBendValue = -1.0 -> +1.0, 0 is at center by using only -8191 -> +8191
	virtual bool __stdcall midiPitchBend(UINT uChannel, int nActualPitchBendValue, float fNormalizedPitchBendValue);

	// 14. MIDI Timing Clock (Sunk to BPM) function called once per clock
	virtual bool __stdcall midiClock();


	// 15. all MIDI messages -
	// NOTE: set m_bWantAllMIDIMessages true to get everything else (other than note on/off)
	virtual bool __stdcall midiMessage(unsigned char cChannel, unsigned char cStatus, unsigned char cData1, unsigned char cData2);

	// 16. initUI() is called only once from the constructor; you do not need to write or call it. Do NOT modify this function
	virtual bool __stdcall initUI();



	// Add your code here: ----------------------------------------------------------- //
	
	//sin source wavetable
	float m_fSinArray[1024];

	//variables
	float m_fReadIndex;
	float m_fQuadPhaseReadIndex;
	float m_f_inc;
	float m_fPDInc;
	float m_fPDReadIndex;
	float m_fLastSample;
	float m_fStartingHeight;

	bool m_bNoteOn;

	//functions
	void cookSlope();

	float doPDBLEP(float fOutSample)
	{
		if(m_uBLEP == BLEP4)
		{
			return fOutSample + doBLEP_N(&dBLEPTable_8_BLKHAR[0], /* BLEP table */
					4096,			/* BLEP table length */
					m_fReadIndex/1024.0,		/* current phase value */
					fabs(m_fPDInc/1024.0),	/* abs(dInc) is for FM synthesis with negative frequencies */
					m_fStartingHeight,			/* sawtooth edge height = 1.0 */
					false,		/* falling edge */
					4,			/* 1 point per side */
					false);		/* no interpolation */
		}

		if(m_uBLEP == BLEP8)
		{
			return fOutSample + doBLEP_N(&dBLEPTable_8_BLKHAR[0], /* BLEP table */
					4096,			/* BLEP table length */
					m_fReadIndex/1024.0,		/* current phase value */
					fabs(m_fPDInc/1024.0),	/* abs(dInc) is for FM synthesis with negative frequencies */
					m_fStartingHeight,			/* sawtooth edge height = 1.0 */
					false,		/* falling edge */
					8,			/* 1 point per side */
					false);		/* no interpolation */
		}

		if(m_uBLEP == PB2)
		{
			return fOutSample + doPolyBLEP_2(m_fReadIndex/1024.0, 
					 abs(m_fPDInc/1024.0),/* abs(dInc) is for FM synthesis with negative frequencies */
					 1.0,		/* sawtooth edge = 1.0 */
					 false);	/* falling edge */
		}

		else
			return fOutSample;
	}

	void cookFrequency()
	{
		switch(m_uIncType)
		{
			case NORM:
			{
				// inc = L*fd/fs
				m_f_inc = 1024.0*m_fFrequency_Hz/(float)m_nSampleRate;
				break;
			}

			/*case PIECEWISE2:
			{
				if(m_fTotalInc < (m_fKnee1/100.0)*1024)
				{
					m_f_inc = (1024.0*m_fFrequency_Hz/(float)m_nSampleRate)*m_fPiecewiseSlope1;
					m_fTotalInc += m_f_inc;
				}
				else
				{
					m_f_inc = (1024.0*m_fFrequency_Hz/(float)m_nSampleRate)*(1.0/m_fPiecewiseSlope1);
					m_fTotalInc += m_f_inc;
				}
				if(m_fTotalInc >= 1024.0)
					m_fTotalInc = 0.0;
				break;
			}

			case PIECEWISE3:
			{
				if(m_fTotalInc < m_fKnee1*10.24)
				{
					m_f_inc = (1024.0*m_fFrequency_Hz/(float)m_nSampleRate)*m_fPiecewiseSlope1;
					m_fTotalInc += m_f_inc;
				}
				else if(m_fTotalInc < m_fKnee2*10.24)
				{
					m_f_inc = (1024.0*m_fFrequency_Hz/(float)m_nSampleRate)*(m_fPiecewiseSlope2);
					m_fTotalInc += m_f_inc;
				}
				else
				{
					m_f_inc = (1024.0*m_fFrequency_Hz/(float)m_nSampleRate)*(m_fPiecewiseSlope3);
					m_fTotalInc += m_f_inc;
				}
				if(m_fTotalInc >= 1024.0)
					m_fTotalInc = 0.0;

				break;
			}*/

			case COSINE:
			{
				//sin(omega*t + phi)
				//omega = m_fFrequency_Hz/m_nSampleRate
				//t = m_fReadIndex/1024.0
				//phi = 0
				m_f_inc = 1024.0*cos( (float)(m_fFrequency_Hz/(float)m_nSampleRate)*(2*pi)*(m_fReadIndex/1024.0) );
				break;
			}
			case RAND:
			{
				float random_value = fmod((float)(11.0*rand()/10.0), m_fUpperLimit);
				if(rand()%3 == 0)
					random_value *= -1;
				m_f_inc = m_f_inc = (1024.0*m_fFrequency_Hz/(float)m_nSampleRate) + random_value;
				break;
			}

			default:
			{
				m_f_inc = 1024.0*m_fFrequency_Hz/(float)m_nSampleRate;
				break;
			}
		}
	}

	void reset()
	{
		m_fReadIndex = 0.0;
		m_fQuadPhaseReadIndex = 256.0;	// 1/4 of our 1024 point buffer
	}

	void doOscillate(float* pYn, float* pYqn);


	// END OF USER CODE -------------------------------------------------------------- //


	// ADDED BY RACKAFX -- DO NOT EDIT THIS CODE!!! ----------------------------------- //
	//  **--0x07FD--**

	float m_fFrequency_Hz;
	UINT m_uIncType;
	enum{NORM,PIECEWISE2,PIECEWISE3,PIECEWISE4,COSINE,RAND};
	float m_fRise1;
	float m_fRun1;
	float m_fKnee2Height;
	float m_fKnee2;
	float m_fThree_FourSplit;
	float m_fUpperLimit;
	UINT m_uBLEP;
	enum{NONE,BLEP4,BLEP8,PB2};

	// **--0x1A7F--**
	// ------------------------------------------------------------------------------- //

};






















































































































